From 80ad95e37a6f86c7d3f679f2fcb95bf78ba494a7 Mon Sep 17 00:00:00 2001
From: Michele Baldessari <michele@acksyn.org>
Date: Sun, 18 Feb 2018 22:20:47 +0100
Subject: [PATCH] Return changed exit code with --noop and count(out_of_sync)
 resources > 0

This should address the following bug:
https://tickets.puppetlabs.com/browse/PUP-686

When puppet is called with --noop all resources are noop-ed except thos
that have an explicit 'noop => false'. Let's simply add the changes
status bit (0x2) if --noop was used and there are out of sync resources.
Handle all other resources as previously, so that if some resources were
indeed executed we set the return code as usual.
---
 lib/puppet/application/apply.rb  | 6 ++++++
 lib/puppet/transaction/report.rb | 3 +++
 2 files changed, 9 insertions(+)

diff --git a/lib/puppet/application/apply.rb b/lib/puppet/application/apply.rb
index 6b96c82db35a..4e7e8eb0732d 100644
--- a/lib/puppet/application/apply.rb
+++ b/lib/puppet/application/apply.rb
@@ -101,6 +101,12 @@ configuration options can also be generated by running puppet with
 
   6: The run succeeded, and included both changes and failures.
 
+  When running with --noop (and no resources have been explicitely set to noop => false)
+  it will return 0 for no changes (the system would already be in the desired state)
+  and it will return 2 if changes would be needed. If there are resources that explicitely
+  have noop => false (and hence will effectively run), they will affect the return code
+  just normally.
+
 * --help:
   Print this help message
 
diff --git a/lib/puppet/transaction/report.rb b/lib/puppet/transaction/report.rb
index 1f4234de8488..d78216d77994 100644
--- a/lib/puppet/transaction/report.rb
+++ b/lib/puppet/transaction/report.rb
@@ -393,6 +393,9 @@ class Puppet::Transaction::Report
   #
   def exit_status
     status = 0
+    if @noop && @metrics["resources"] && @metrics["resources"]["out_of_sync"]
+      status |= 2 if @metrics["resources"]["out_of_sync"] > 0
+    end
     if @metrics["changes"] && @metrics["changes"]["total"] &&
         @metrics["resources"] && @metrics["resources"]["failed"] &&
         @metrics["resources"]["failed_to_restart"]
-- 
2.14.3

